/*
 * Created on 05.07.2004
 */
package delayeffect;

import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.sound.sampled.*;
import java.io.*;
import java.awt.*;

/**
 * @author jl
 */
public class DelayEffect extends JFrame {

	JSlider delaySlider;
	JSlider feedbackSlider;
	JSlider dryWetSlider;
	JSlider modulationDepthSlider;
	JSlider modulationRateSlider;

	public static void main(String[] args) {
		DelayEffect myFrame = new DelayEffect();	
	}
	
	private DelayEffect()
	{
		getContentPane().setLayout(new GridLayout(6, 2));
		AudioAdapter aa = new AudioAdapter(this);
		
		JButton playButton = new JButton("Play");
		playButton.addActionListener(aa);
		getContentPane().add(playButton);
		JButton stopButton = new JButton("Stop");
		stopButton.addActionListener(aa);
		getContentPane().add(stopButton);
		delaySlider = new JSlider(0, 500, 200);
		delaySlider.addChangeListener(aa);
		getContentPane().add(new JLabel("Delay Time", JLabel.RIGHT));
		getContentPane().add(delaySlider);
		feedbackSlider = new JSlider(0, 100, 50);
		feedbackSlider.addChangeListener(aa);
		getContentPane().add(new JLabel("Feedback Level", JLabel.RIGHT));
		getContentPane().add(feedbackSlider);
		dryWetSlider = new JSlider(0, 100, 30);
		dryWetSlider.addChangeListener(aa);
		getContentPane().add(new JLabel("Dry/Wet", JLabel.RIGHT));
		getContentPane().add(dryWetSlider);		
		modulationDepthSlider = new JSlider(0, 100, 10);
		modulationDepthSlider.addChangeListener(aa);
		getContentPane().add(new JLabel("Modulation Depth", JLabel.RIGHT));
		getContentPane().add(modulationDepthSlider);
		modulationRateSlider = new JSlider(0, 30, 10);
		modulationRateSlider.addChangeListener(aa);
		getContentPane().add(new JLabel("Modulation Rate", JLabel.RIGHT));
		getContentPane().add(modulationRateSlider);
								
		pack();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);		
		show();
	}
}

class AudioAdapter
implements ActionListener, ChangeListener
{
	private DelayEffect de;
	
	AudioAdapter(DelayEffect de)
	{
		this.de = de;
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if(e.getActionCommand().equals("Play"))
		{
			AudioThread.stopRequested = false;
			new AudioThread().start();
		}
		else if(e.getActionCommand().equals("Stop"))
			AudioThread.stopRequested = true;
	}
	
	public void stateChanged(ChangeEvent e)
	{
		JSlider s = (JSlider)e.getSource();
		if(s == de.delaySlider)
		{
			AudioThread.delay = s.getValue();
		}
		else if(s == de.feedbackSlider)
		{
			AudioThread.feedback = s.getValue();
		}
		else if(s == de.dryWetSlider)
		{
			AudioThread.dryWet = s.getValue();
		}
		else if(s == de.modulationDepthSlider)
		{
			AudioThread.modulationDepth = s.getValue();
		}
		else if(s == de.modulationRateSlider)
		{
			AudioThread.modulationRate = s.getValue();
		}		
	}
}

class AudioThread extends Thread
{
	static volatile boolean stopRequested;
	static volatile int delay = 200; // ms
	static volatile int feedback = 50; // percent
	static volatile int dryWet = 30; // percent
	static volatile int modulationDepth = 10; // 0.1 percent
	static volatile int modulationRate = 10; // 0.1 Hz
		
	public void run()
	{		
		byte[] bufferIn = new byte[2000];
		byte[] bufferOut = new byte[2000];
		
		final int delayBufferSize = 40000;
		double[][] delayBuffer = new double[delayBufferSize][2];

		try
		{
			File inFile = new File("C:\\WINDOWS\\Media\\Windows XP-Startvorgang.wav");
			AudioFileFormat aff = AudioSystem.getAudioFileFormat(inFile);
			AudioFormat af = aff.getFormat();
			if(af.isBigEndian())
				throw new Exception("no litte endian");
			if(af.getChannels() != 2)
				throw new Exception("no stereo file");
			if(af.getSampleSizeInBits() != 16)
				throw new Exception("no 16 bit file");
				
			AudioInputStream inStream = AudioSystem.getAudioInputStream(inFile);				
	
			DataLine.Info lineInfo = new DataLine.Info(SourceDataLine.class, af);
			SourceDataLine myLine = (SourceDataLine)AudioSystem.getLine(lineInfo);
			myLine.open(af, 20000);		
			myLine.start();
			
			int currentSample = 0;
			int inPointer = 0;
			
			while(!stopRequested)
			{
				int numBytesRead = inStream.read(bufferIn);
				if(numBytesRead <= 0)
					break;
								
				int pos = 0;
				for(int i = 0; i < numBytesRead/4; i++)
				{
					double time = currentSample/af.getFrameRate();
					currentSample++;
					double delayTime = 0.001*delay*
						(1.0 + 0.001*modulationDepth*
							Math.sin(2.0*Math.PI*0.1*modulationRate*time)
						);
					double offset = delayTime*af.getFrameRate();	
					double outPointer = (inPointer - offset + delayBufferSize)%delayBufferSize;
					int oPFloor = (int)Math.floor(outPointer);
					int oPCeil = oPFloor + 1;
					if(oPCeil >= delayBufferSize)
						oPCeil = 0;

					// set mix to 0 to hear the effect
					// of applying no interpolation
					double mix = outPointer - oPFloor;

					for(int ch = 0; ch < 2; ch++)
					{
						int oldPos = pos;
						int lo = bufferIn[pos++];
						int hi = bufferIn[pos++];
						short v = (short)((hi << 8) | (lo & 0xFF));
						double in = v/(double)(Short.MAX_VALUE+1);
							
						double delayOut =
							(1.0-mix)*delayBuffer[oPFloor][ch]
							+ mix*delayBuffer[oPCeil][ch];
						delayBuffer[inPointer][ch] = in + 0.01*feedback*delayOut;
						double out = (1.0-0.01*dryWet)*in + 0.01*dryWet*delayOut;
						out *= 0.2/(0.2+0.01*feedback); // to reduce the output level for high feedback
				
						short w = (short)(out*Short.MAX_VALUE);
						bufferOut[oldPos++] = (byte)(w & 0xFF);
						bufferOut[oldPos] = (byte)(w >> 8);
					}
					inPointer++;
					if(inPointer >= delayBufferSize)
						inPointer = 0;
				}
				myLine.write(bufferOut, 0, numBytesRead);
			}
			myLine.drain();
			myLine.stop();
			inStream.close();
		}
		catch(Exception e)
		{
			throw new RuntimeException(e);
		}		
	}
}
